proc InitGIDProject { dir } {
    set ::FluentPriv(dir) $dir
    set ::FluentPriv(version) 1.0
  
    fluent::splash $dir 1
    fluent::modifymenus

    fluent::MyBitmaps $dir

    set ::FluentPriv(CalculationFileExtension) $::GidPriv(CalculationFileExtension)
    set ::GidPriv(CalculationFileExtension) ".msh"
}

proc EndGIDProject {} {
    fluent::EndMyBitmaps
    set ::GidPriv(CalculationFileExtension) $::FluentPriv(CalculationFileExtension)
}

proc ChangedLanguage { newlanguange } {
    #to force refresh some menus managed by the problemtype when the user change to newlanguage
    fluent::modifymenus
}

namespace eval fluent {}

proc fluent::splash { dir autodestroy } {
    
    if { [::GidUtils::VersionCmp 9.1.0b] >= 0 } {
	set text "Version $::FluentPriv(version)"
	GidUtils::Splash [file join $dir images fluent_about.gif] \
	    .splash $autodestroy [list $text 70 300]
    } else {
	#code unnecesary form GiD 9.1.0b, GidUtils::Splash can be used
	set VersionNumber $::FluentPriv(version)
	variable splashshown
	
	if { [.gid.central.s disable windows] } { return }
	
	if { [info exists splashshown] && $autodestroy } { return }
	if { [ winfo exist .splash]} {
	    destroy .splash
	    update
	}
	set splashshown 1
	
	toplevel .splash
	wm withdraw .splash
	
	set im [image create photo -file [file join $dir images fluent_about.gif]]
	set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
	set y [expr [winfo screenheight .splash]/2-[image height $im]/2]

	wm geom .splash +$x+$y
	wm transient .splash .gid
	wm overrideredirect .splash 1
	pack [label .splash.l -image $im -relief ridge -bd 2]
	
	label .splash.lv -text "Version $VersionNumber" -font "times 32"
	place .splash.lv -x 70 -y 300
	bind .splash <1> "destroy .splash"
	bind .splash <KeyPress> "destroy .splash"
	grab .splash
	focus .splash
	update
	wm deiconify .splash
	raise .splash .gid
	update
	if { $autodestroy } {
	    after 2000 "if { [ winfo exist .splash] } { 
	    destroy .splash
	}"
	}
    }
}

proc fluent::modifymenus { } {
    set dir $::FluentPriv(dir)
    foreach where {PRE POST} {
	GiDMenu::InsertOption "Help#C#menu" [list [= "FLUENT interface help"]] 0 $where \
	    [list GiDCustomHelp -dir [file join $dir info] \
		 -title [= "FLUENT interface help"] -report 0] \
	    "" "" "insert" =
	GiDMenu::InsertOption "Help#C#menu" [list [= "About FLUENT interface"]] end $where \
	    [list fluent::splash $dir 0] "" "" "insertafter" =
    }
    
    GidChangeDataLabel "Problem Data" [= "Problem Dimension"]
    
    #We hide one of the conditions set    
    
    set problem_dimension [string index [GiD_AccessValue get gendata Problem_dimension] 0] 
    
    switch $problem_dimension {
	2 {
	    set normal_book "2D"
	    set hidden_book "3D"
	}
	3 {
	    set normal_book "3D"
	    set hidden_book "2D"
	}     
    }    
    
    GiD_ShowBook conditions $hidden_book 0   
    GiD_ShowBook conditions $normal_book 1   
    GiDMenu::UpdateMenus
}

proc fluent::MyBitmaps { dir { type "DEFAULT INSIDELEFT"} } {

    global FluentBitmapsNames FluentBitmapsCommands FluentBitmapsHelp ProblemTypePriv

    set FluentBitmapsNames(0) "images/section.gif images/calc.gif"    
    
    set problem_dimension [string index [GiD_AccessValue get gendata Problem_dimension] 0] 
    
    switch $problem_dimension {
	2 {
	    set normal_book "2D"
	}
	3 {
	    set normal_book "3D"
	}     
    }      
    
    set FluentBitmapsCommands(0) [list [list -np- GidOpenConditions $normal_book] \
	    "Files WriteCalcFile"]

    set FluentBitmapsHelp(0) [list [= "Define Boundary Conditions and Continuum zones"] [= "Output file"]]

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always

    set prefix Pre

    set ProblemTypePriv(toolbarwin) [CreateOtherBitmaps FluentBar [= "FLUENT bar"] \
	    FluentBitmapsNames FluentBitmapsCommands \
	    FluentBitmapsHelp $dir "fluent::MyBitmaps [list $dir]" $type $prefix]
    AddNewToolbar "FLUENT bar" ${prefix}FluentBarWindowGeom \
	"fluent::MyBitmaps [list $dir]" [= "FLUENT bar"]
}

proc fluent::EndMyBitmaps {} {
    global ProblemTypePriv
    
#     global BitMapsNames BitMapsCommands BitmapsHelp
#     set BitMapsNames(0) [lreplace $BitMapsNames(0) 0 2]
#     set BitMapsCommands(0) [lreplace $BitMapsCommands(0) 0 2]
#     set BitmapsHelp(0) [lreplace $BitmapsHelp(0) 0 2]
#     return

    ReleaseToolbar "FLUENT bar"
    #rename MyBitmaps ""
    catch { destroy $ProblemTypePriv(toolbarwin) }
}

proc fluent::writefile {} {
    set err [catch { fluent::_writefile } string]
    if { $err } { WarnWin $string }
    return $string
}


proc fluent::_writefile {} {  
  
    set num_zones 0
    set dict_zones ""
    
    #Writing General Data++++++++++++++++++++++++++++++++++++++++
    
    set _ ""
    append _ "(1 \"Created by GiD on [clock format [clock seconds] -format %d-%h-%y]"
    append _ " at [clock format [clock seconds] -format %H:%M:%S]\")"
   
    set problem_dimension [string index [GiD_AccessValue get gendata Problem_dimension] 0] 
    append _ "\n(0 \"Dimension:\")"  
    append _ "\n(2 $problem_dimension)" 
    
    #Writing nodes(ZONE=1)++++++++++++++++++++++++++++++++++++++++
    
    append _ "\n(0 \"Nodes:\")"
    incr num_zones    
    
    #We save enough space for all the nodes 

    append _ "\n(10 (0 1 [format %x [GiD_Info Mesh MaxNumNodes]] 0 $problem_dimension))"
    
    #We enter the information for each node
    
    foreach "n x y z" [GiD_Info Mesh Nodes] {
	append _ "\n(10 ([format %x $num_zones] [format %x $n] [format %x $n] 1 $problem_dimension)("
	switch $problem_dimension {
	    2 {
		append _ "\n[format "%16e %16e" $x $y]\n))"
	    }
	    3 {
		append _ "\n[format "%16e %16e %16e" $x $y $z]\n))"
	    }   
	}        
    }    
     
    # We write as virtual nodes the undefined nodes
    
    for {set i 1} {$i<=[GiD_Info Mesh MaxNumNodes]} {incr i} {
	if {[GiD_Info Mesh Nodes $i $i] eq ""} {
	    append _ "\n(10 ([format %x $num_zones] [format %x $i] [format %x $i] 0 $problem_dimension)(" 
	    switch $problem_dimension {
		2 { append _ "\n 0.00000000e+00  0.00000000e+00\n))" }
		3 { append _ "\n 0.00000000e+00  0.00000000e+00  0.00000000e+00\n))" }
	    }  
	}        
    }
    
    #Writing Cells+++++++++++++++++++++++++++++++
    
    append _ "\n(0 \"Cells:\")" 
    
    #We save enough space for all the cells
    
    append _ "\n(12 (0 1 [format %x [GiD_Info Mesh NumElements]] 0))" 
    
    #It's not possible to skip elements keeping the proper zone as done with the nodes.
    #That's why an dict_elements with the real element number is created. This simplification is very important in terms of zone spliting for FLUENT    
       
    array unset contset     
    set dict_elements ""
    
    #We construct conset array with the contiuum conditions info
    
    foreach cnd_type [list Surface_Continuum Volume_Continuum] {
	foreach cnd [GiD_Info conditions $cnd_type mesh] {
	    lappend contset([lindex $cnd 4]) [list [lindex $cnd 1] [lindex $cnd 3]]
	}      
    }
    
    #We construct the default array
    
    switch $problem_dimension {
	2 {
	    set elemtype_list [list Triangle Quadrilateral]  
	    set cnd_type Surface_Continuum          
	}
	3 {
	    set elemtype_list [list Triangle Quadrilateral Tetrahedra Hexahedra Prism] 
	    set cnd_type Volume_Continuum 
	}      
    }
     
    for {set i 1} {$i<=[GiD_Info Mesh MaxNumElements]} {incr i} {     
	foreach elemtype $elemtype_list {
	    if {[GiD_Info Mesh Elements $elemtype $i $i] ne "" } {              
		if {[lindex [lindex [GiD_Info Conditions $cnd_type mesh $i]  0 ] 0] ne "E"} {
		    #By default, we consider a fluid cell
		    lappend contset(Default_continuum) [list $i Fluid]
		}                  
	    }          
	}         
    }
    
    #And now we print all continuum zones
    
    set num_elems 0
    
    foreach cont_group [array names contset] {
	
	set cont_info [lsort -unique $contset($cont_group)]  
	set printed_caption 0
	
	foreach cell $cont_info {        
	    
	    incr num_elems
	    set j 0
	    
	    foreach k [list real_elementid type] {
		set $k [lindex $cell $j]
		incr j   
	    }
	    
	    dict set dict_elements $real_elementid $num_elems
	    
	    if {$printed_caption eq "0"} {
		set original_type $type                
		set translated_type 0
		switch $original_type {
		    Solid {
		        set translated_type 11
		    }
		    Fluid {
		        set translated_type 1
		    }   
		}
		#Caption printing
		incr num_zones
		dict set dict_zones $cont_group ZONE_ID $num_zones 
		dict set dict_zones $cont_group ZONE_TYPE [fluent_cond_name $original_type]                   
		append _ "\n(0 \"Continuum $num_zones : $cont_group\")"
		append _ "\n(12 ([format %x $num_zones] [format %x $num_elems]" 
		set final_elementid [expr ($num_elems+[llength $cont_info]-1)]                
		append _ " [format %x $final_elementid] $translated_type 0)(\n"   
		set printed_caption 1
	    }
	    
	    if {$type ne $original_type} {
		error [= "Cell %s of continuum set %s has a continuum type not according with the rest of the elements from its condition. Please correct this" $real_elementid $cont_group]
	    }
		     
	    foreach elemtype [list Triangle Tetrahedra Quadrilateral Hexahedra Prism] element_ident [list 1 2 3 4 6] {
		if {[GiD_Info Mesh Elements $elemtype $real_elementid $real_elementid] ne "" } {                
		    append _ "$element_ident "                     
		}          
	    }
	}
	append _ "\n))"   
    }
    
    array unset contset
    
    #Writing faces++++++++++++++++++++++++++++++++++++++++ 
    
    set faces_distribution ""
    
    dict set faces_distribution Triangle  [list [list 1 2] [list 2 3] [list 3 1]]
    dict set faces_distribution Quadrilateral [list [list 1 2] [list 2 3] [list 3 4] [list 4 1]]
    dict set faces_distribution Tetrahedra [list [list 1 2 3] [list 2 3 4] [list 3 4 1] [list 4 2 1]]
    dict set faces_distribution Hexahedra [list [list 1 2 3 4] [list 1 4 8 5] [list 1 5 6 2] [list 2 6 7 3] [list 3 7 8 4] [list 5 8 7 6]]
    dict set faces_distribution Prism [list [list 1 2 3] [list 1 4 5 2] [list 2 5 6 3] [list 3 6 4 1] [list 4 6 5]]
    
    if {[GiD_Info Project Quadratic]=="1"} {error [= "This interface does not support quadratic elements"]}  

    foreach cnd_type [list Line_Boundary_conditions Surface_Boundary_conditions] {
	foreach cnd [GiD_Info conditions $cnd_type mesh] {
	    lappend faceset([lindex $cnd 4]) [list [lindex $cnd 0] [lindex $cnd 1] [fluent_cond_number [lindex $cnd 3]]]
	}      
    }
       
    #First we create our array relating each face with the positive and negative element    
    
    array unset face_to_elements   
    
    foreach elemtype [list Triangle Quadrilateral Tetrahedra Hexahedra Prism] maxfaces [list 3 4 4 6 5] {        
	foreach element_line [GiD_Info Mesh Elements $elemtype -sublist] {
	    for {set nface 1} {$nface<=$maxfaces} {incr nface} {
		set face_connectivities [extract_connectivities $element_line $elemtype $nface $faces_distribution]
		set found 0
		
		#If found, we add the studied elements to the element connection list of the axisting array entry
		
		foreach permutation_face [give_permutation_faces $face_connectivities] {
		    set face_name [give_hexadecimal_name $permutation_face]
		    if {[info exists face_to_elements($face_name)]} {
		        lappend face_to_elements($face_name) [lindex $element_line 0]
		        set found 1
		    }                    
		}    
	    
		#If the face wasn't saved, we take the positive orientation as the actual and we 
		#append the studied element as the first in the list. First index indicate that the position is not already printed
	    
		if {$found eq "0"} {
		    lappend face_to_elements([give_hexadecimal_name $face_connectivities]) 0
		    lappend face_to_elements([give_hexadecimal_name $face_connectivities]) [lindex $element_line 0]                
		}
	    }            
	} 
    }
    
    #General face caption
    
    if {[array size face_to_elements] ne "0"} {
	append _ "\n(0 \"Faces:\")" 
	append _ "\n(13 (0 [format %x 1] [format %x [array size face_to_elements]] 0))"        
    }

    #Printing each face
    
    set num_faces 0
    
    foreach i [array names faceset] {
	set face_info [lsort -unique $faceset($i)]      
	set printed_caption 0    
	
	foreach face $face_info {
	    
	    set j 0
	    
	    foreach k [list elem face_ident type] {
		set $k [lindex $face $j]
		incr j   
	    }
	    
	    if {$printed_caption eq "0"} {
		set original_type $type
		#Caption printing
		incr num_zones
		dict set dict_zones $i ZONE_ID $num_zones 
		dict set dict_zones $i ZONE_TYPE [fluent_cond_name $original_type] 
		append _ "\n(0 \"Face $num_zones : $i\")"
		incr num_faces
		append _ "\n(13 ([format %x $num_zones] [format %x $num_faces]" 
		incr num_faces [expr ([llength $face_info]-1)]
		append _ " [format %x $num_faces] [format %x $original_type] 0)("   
		set printed_caption 1
	    }
	    
	    if {$type ne $original_type} {
		error [= "Element %s of face set $i has a bounbary type not according with the rest of the elements from its condition. Please correct this" $elem]
	    }
	    
	    set connectivities ""
	    
	    foreach elemtype [list Triangle Quadrilateral Tetrahedra Hexahedra Prism] {
		set element_line [GiD_Info mesh Elements $elemtype $elem $elem]
		if {$element_line ne ""} {
		    set connectivities [extract_connectivities $element_line $elemtype $face_ident $faces_distribution]
		    break
		}
	    }
	    
	    #Printing each face line
	    append _ "\n[llength $connectivities]" 
	    
	    foreach connectivity_node $connectivities {
		append _ " [format %x $connectivity_node]"
	    }   
	    
	    #We always append the right element, which is the studied element. We use the elements dict to find the real value
	    set real_elem [dict get $dict_elements $elem]
	    append _ " [format %x $real_elem]"
	    
	    #We found the element in our array
	    
	    set face_name [give_hexadecimal_name $connectivities]
	    set found_left 0
	    
	    if {[info exists face_to_elements($face_name)]} {
		#We mark as printed this face in the array
		lset face_to_elements($face_name) 0 "1"
		#Then we write the left cell if needed 
		if {[llength $face_to_elements($face_name)] eq "3"} {                 
		    if {[lindex $face_to_elements($face_name) 2] eq $elem} {
		        set real_elem [dict get $dict_elements [lindex $face_to_elements($face_name) 1]]
		    } else {
		        set real_elem [dict get $dict_elements [lindex $face_to_elements($face_name) 2]]
		    }
		    append _ " [format %x $real_elem]"
		    set found_left 1
		} 
	    } else {
		foreach permutation_face [give_permutation_faces $connectivities] {             
		    set face_name [give_hexadecimal_name $permutation_face]
		    if {[info exists face_to_elements($face_name)]} {
		        #We mark as printed this face in the array
		        lset face_to_elements($face_name) 0 "1"
		        #Then we write the left cell if needed  
		        if {[llength $face_to_elements($face_name)] eq "3"} {
		            if {[lindex $face_to_elements($face_name) 2] eq $elem} {
		                set real_elem [dict get $dict_elements [lindex $face_to_elements($face_name) 1]]
		            } else {
		                set real_elem [dict get $dict_elements [lindex $face_to_elements($face_name) 2]]
		            } 
		            append _ " [format %x $real_elem]"
		            set found_left 1                                               
		        } 
		        break                    
		    }     
		}
	    }
	    
	    #If we don't find a left cell we just print 0
	    
	    if {$found_left eq "0"} {append _ " 0"}  
	}
	
	append _ "\n))" 
    }
    
    #Non printed faces will be printed as interior faces or walls:
    
    set interior_faces 0
    set wall_faces 0
    
    foreach face [array names face_to_elements] {
	if {([lindex $face_to_elements($face) 0] eq "0")} {
	    switch [llength $face_to_elements($face)] {
		3 {
		    incr interior_faces
		}
		2 {
		    incr wall_faces
		}
		default {}
	    }
	}
    }
    
    #Default interior zone
    
    if {$interior_faces > "0"} {
	incr num_zones
	dict set dict_zones Default_interior_zone ZONE_ID $num_zones 
	dict set dict_zones Default_interior_zone ZONE_TYPE interior
	append _ "\n(0 \"Face [format %x $num_zones] : Default_interior_zone\")"
	append _ "\n(13 ([format %x $num_zones] [format %x [expr ($num_faces+1)]] [format %x [expr ($num_faces+$interior_faces)]] 2 0)("
	
	foreach face [array names face_to_elements] {
	    if {([lindex $face_to_elements($face) 0] eq "0") && ([llength $face_to_elements($face)] eq "3")} {
		set connectivities [split $face ":"]
		append _ "\n[llength $connectivities]"                
		foreach node $connectivities { append _ " $node" }
		set real_elem1 [dict get $dict_elements [lindex $face_to_elements($face) 1]]
		set real_elem2 [dict get $dict_elements [lindex $face_to_elements($face) 2]]
		append _ " [format %x $real_elem1] [format %x $real_elem2]"
	    }
	}
	
	append _ "))" 
    }
    
    #Default Wall zone    
    
    if {$wall_faces > "0"} {
	incr num_zones
	dict set dict_zones Default_wall_zone ZONE_ID $num_zones 
	dict set dict_zones Default_wall_zone ZONE_TYPE wall 
	append _ "\n(0 \"Face [format %x $num_zones] : Default_wall_zone\")"
	append _ "\n(13 ([format %x $num_zones] [format %x [expr ($num_faces+$interior_faces+1)]] [format %x [expr ($num_faces+$interior_faces+$wall_faces)]] 3 0)("
	
	foreach face [array names face_to_elements] {
	    if {([lindex $face_to_elements($face) 0] eq "0") && ([llength $face_to_elements($face)] eq "2")} {
		set connectivities [split $face ":"]
		append _ "\n[llength $connectivities]"                
		foreach node $connectivities { append _ " $node" }
		set real_elem [dict get $dict_elements [lindex $face_to_elements($face) 1]]
		append _ " [format %x $real_elem] 0"
	    }
	}
	
	append _ "))" 
    }
    
    #ZONES DEFINITION++++++++++
    
    append _ "\n(0 \"Zones definition:\")"
    
    dict for {id info} $dict_zones {      
	dict with info {
	    append _ "\n(45 ($ZONE_ID $ZONE_TYPE $id)())"
	}
    }
    
    array unset face_to_elements
    set dict_elements ""
    set dict_zones ""
    

    return $_
}

proc fluent::ComunicateWithGiD { op args } {
    
    variable SetName
    
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    
	    set f [frame $PARENT.f]
	    
	    set values ""
	    
	    set condname [lindex [split $STRUCT ","] 1] 
	    
	    
	    if { [string match *Boundary $condname] } {
		set initial_list {Cond1 Cond2 Cond3 Cond4}
	    } elseif { [string match *Continuum $condname] } {
		set initial_list {Cont1 Cont2 Cont3 Cont4}
	    } else {
		set initial_list {Cond1 Cond2 Cond3 Cond4}
	    }
	    
	    foreach domain [list geometry mesh] {
		foreach cnd [GiD_Info conditions $condname $domain] {
		    set name [lindex $cnd 4]
		    if { [lsearch $values $name] == -1 && [lsearch $initial_list $name] == -1 } { 
		        lappend values $name 
		    }
		}
	    }
	    
	    if { [llength $values] } { lappend values --- }            
	    append values " $initial_list"
	    
	    label $f.l1 -text [= "Name"]
	    if { ![info exists SetName] || [string trim $SetName] eq "" } {
		set SetName [lindex $values 0]
	    }
	    ComboBox $f.cb1 -textvariable fluent::SetName -values $values
	    #$f.cb1 setvalue first
	    after 100 Widget::traverseTo $f.cb1
	    
	    grid $f.l1 $f.cb1 -sticky nw
	    grid configure $f.cb1 -sticky new -padx "1 5"
	    grid rowconfigure $f 1 -weight 1
	    grid columnconfigure $f 1 -weight 1
	    
	    GidHelpRecursive $f [= "Enter the name for the condition"]
	    grid $f -row $ROW -column 0 -sticky nsew -columnspan 2 -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1
	    return ""
	}
	"SYNC" {           
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]
	    
	    set SetName [string trim $SetName]
	    if { $SetName eq "" } {
		return [list ERROR [= "Name is not valid"]]
	    } elseif {[string length $SetName]>"80"} {
		return [list ERROR [= "Name is too long"]]
	    } else {                
		DWLocalSetValue $GDN $STRUCT Name $SetName
	    }
	    return ""
	}
    }
}

proc fluent::Update_conditions { op args } {    
    switch $op {        
	"SYNC" {   
	    set problem_dimension [string index [GiD_AccessValue get gendata Problem_dimension] 0]             
	    switch $problem_dimension {
		2 {
		    set normal_book "2D"
		    set hidden_book "3D"
		}
		3 {
		    set normal_book "3D"
		    set hidden_book "2D"
		}     
	    }    
	    
	    GiD_ShowBook conditions $normal_book 1    
	    GiD_ShowBook conditions $hidden_book 0 
	    
	    fluent::MyBitmaps [GiD_Info problemtypepath]
	    GiDMenu::UpdateMenus
	}          
    } 
}

proc fluent::fluent_cond_number { real_name } {
    
    set cond_list [list Wall Pressure-Inlet Inlet-Vent Intake-fan Pressure-Outlet Exhaust-Fan Oulet-Vent Symmetry Periodic-Shadow Pressure-far-field Velocity-Inlet Periodic Fan Porous-Jump Radiator Mass-flow-inlet Interface Parent Outflow Axis]
    set cond_ident [list 3 4 4 4 5 5 5 7 8 9 10 12 14 14 14 20 24 31 36 37]
    
    set position [lsearch $cond_list $real_name]
    
    if { $position ne "-1"} {
	return [lindex $cond_ident $position]
    }
    
    return -1;     
}

proc fluent::fluent_cond_name { real_name } {
    
    set cond_list [list Fluid Solid 3 4 4 4 5 5 5 7 8 9 10 12 14 14 14 20 24 31 36 37]
    set cond_ident [list fluid solid wall pressure-inlet inlet_vent intake-fan pressure-outlet exhaust-fan oulet-vent symmetry periodic-shadow pressure-far-field velocity-inlet periodic fan porous-jump radiator mass-flow-inlet interface parent outflow axis]
    
    set position [lsearch $cond_list $real_name]
    
    if { $position ne "-1"} {
	return [lindex $cond_ident $position]
    }
    
    return -1;     
}

proc fluent::extract_connectivities { element_line elemtype face_ident faces_distribution} { 
    
    # GiD Triangle faces:        12   23   31
    # GiD Quadrilateral faces:   12   23   34   41
    # GiD Tetra faces: 123  243  341  421
    # GiD Hexa faces: 1234  1485  1562  2673  3784  5876
    # GiD Prism faces: 123 1452 2563 3641  465      
    
    set faces_list [lindex [dict get $faces_distribution $elemtype] [expr ($face_ident-1)]]
    
    set nodes_list ""
    
    foreach position $faces_list {
	lappend nodes_list [lindex $element_line $position]          
    }
    
    return $nodes_list
}

proc fluent::give_permutation_faces { items } {
     set l [llength $items]
     if {[llength $items] < 2} {
	return $items
     } else {
	for {set j 0} {$j < $l} {incr j} {
	    foreach subcomb [give_permutation_faces [lreplace $items $j $j]] {
		lappend res [concat [lindex $items $j] $subcomb]
	    }
	}
	return $res
     }
 }


proc fluent::give_hexadecimal_name { face } {
    
    set name ""
    
    for {set i 0} {$i<=[expr ([llength $face]-2)]} {incr i} {
	append name "[format %x [lindex $face $i]]:"     
    }
    
    append name [format %x [lindex $face [expr ([llength $face]-1)]]]
    
    return $name    
}

